/***************************************************************************
 *   Copyright (C) 2005 by Oliver Saal                                     *
 *   http://www.oliver-saal.de/                                            *
 *   osaal@gmx.de                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include "updater.h"
#include "define.h"

#include <qtextstream.h>
#include <qmessagebox.h>
#include <time.h>
#include <qapplication.h>
#include <qurl.h>
#include <qdir.h>
#include <qfile.h>

CUpdater::CUpdater (QWidget *pParent, const char *name) : QObject ((QObject*)pParent, name)
{
	m_pParent = pParent;
	m_iDBVersion = 0;
	m_iSoftwareVersion = 0;
	m_iRequInfoID = 0;
	m_iRequDBID = 0;
	m_uLastUpdateCheck = 0;

	connect (&m_http, SIGNAL (requestFinished (int, bool)), this, SLOT(onHttpRequestFinished (int, bool)));
	connect (&m_http, SIGNAL (responseHeaderReceived (const QHttpResponseHeader &)), this, SLOT(onHttpResponseHeaderReceived (const QHttpResponseHeader &)));
	m_pProgress = NULL;
}

CUpdater::~CUpdater ()
{
}

void CUpdater::onHttpResponseHeaderReceived (const QHttpResponseHeader &resp)
{
	m_response = resp;
}

void CUpdater::InitProgressDialog (QString strLabel)
{
	if (m_pProgress == NULL) m_pProgress = new QProgressDialog (qApp->activeWindow());
	m_pProgress->setLabelText (strLabel);
	m_pProgress->show ();
	connect (&m_http, SIGNAL(dataReadProgress(int, int)), m_pProgress, SLOT(setProgress (int, int)));
	connect (m_pProgress, SIGNAL(canceled()), &m_http, SLOT (abort()));
	qApp->processEvents();
}

void CUpdater::CloseProgressDialog ()
{
	if (m_pProgress == NULL) return;
	delete m_pProgress;
	m_pProgress = NULL;
}

void CUpdater::Abort (void)
{
	m_http.abort();
}

void CUpdater::onHttpRequestFinished (int id, bool bError)
{
	if (bError && m_http.error() == QHttp::Aborted) return;
	if (bError)
	{
		CloseProgressDialog ();
		QMessageBox::critical (qApp->activeWindow(), tr("Error"), tr("Could not download file.\n%1").arg(m_http.errorString()));
		return;
	}
	if (m_response.isValid() && (int)(m_response.statusCode()/100) != 2)
	{
		CloseProgressDialog ();
		QMessageBox::critical (qApp->activeWindow(), tr("Error"), tr("%1").arg(QString (m_http.readAll())));
		return;
	}
	if (id == m_iRequInfoID)
	{
		m_iRequInfoID = 0;
		CloseProgressDialog ();
		if (!ExtractUpdateInfo (m_http.readAll()))
		{
			QMessageBox::critical (qApp->activeWindow(), tr("Error"), tr("Could not read update configuration from\nhttp://www.oliver-saal.de/software/mautrechner/data/update.txt"));
			return;
		}
		m_uLastUpdateCheck = time (NULL);
		emit UpdateInfoReceived ();
	}
	else if (id == m_iRequDBID)
	{
		CloseProgressDialog ();
		m_iRequDBID = 0;
		QDir dir (qApp->applicationDirPath()+"/../data/");
		if (!dir.exists() && !dir.mkdir (dir.absPath()))
		{
			QMessageBox::critical (qApp->activeWindow(), tr("Error"), tr("Could create the directory\n%1").arg (QDir::convertSeparators (dir.absPath())));
			return;
		}
		QString strFile = QDir::convertSeparators (dir.absPath() + "/" + QUrl(m_strDBPath).fileName());
		QFile file (strFile);
		if (file.open (IO_WriteOnly))
		{
			file.writeBlock (m_http.readAll());
			file.close ();
			emit DatabaseReceived (strFile);
		}
		else
		{
			QMessageBox::critical (qApp->activeWindow(), tr("Error"), tr("Could create the file\n%1").arg (strFile));
			return;
		}
	}
}

void CUpdater::GetUpdateInfo (const bool bBackground)
{
QHttpRequestHeader head ("GET", "/software/mautrechner/data/update.txt");
	head.setValue("Host", "www.oliver-saal.de");
	head.setValue ("User-Agent", "MautRechner V" MAUTRECHNER_VERSION_STRING " ["OS_NAME"]");
	head.setValue ("Referer", "http://www.oliver-saal.de/software/mautrechner/");

	if (m_iRequInfoID != 0) 
	{
		emit UpdateInfoReceived();
		return;
	}
	if (!bBackground) InitProgressDialog (tr("Downloading update information..."));
	m_http.setHost ("www.oliver-saal.de");
	m_iRequInfoID = m_http.request (head);
}

void CUpdater::GetDatabase (void)
{
QHttpRequestHeader head ("GET", m_strDBPath);
	head.setValue("Host", m_strDBHost);
	head.setValue ("User-Agent", "MautRechner V" MAUTRECHNER_VERSION_STRING " ["OS_NAME"]");
	head.setValue ("Referer", "http://www.oliver-saal.de/software/mautrechner/");

	InitProgressDialog (tr("Downloading toll tabular..."));

	if (m_iRequDBID != 0) return;
	m_http.setHost (m_strDBHost);
	m_iRequDBID = m_http.request (head);
}

static QString GetValue (QString strLine)
{
	return (strLine.mid (strLine.find('=')+1).stripWhiteSpace());
}

static QString GetKey (QString strLine)
{
	return (strLine.left (strLine.find('=')).stripWhiteSpace());
}

bool CUpdater::ExtractUpdateInfo (QByteArray array)
{
QString strLine;
QTextStream stream (array, IO_ReadOnly);

	while (!stream.atEnd ())
	{
		strLine = stream.readLine ().stripWhiteSpace();
		if (strLine.isEmpty()) continue;
		if (strLine[0] == '#') continue;
		if (strLine.find ('=') < 0) continue;

		if (GetKey (strLine) == "host")	m_strDBHost = GetValue(strLine);
		else if (GetKey (strLine) == "path") m_strDBPath = GetValue(strLine);
		else if (GetKey (strLine) == "version") m_iDBVersion = GetValue(strLine).toInt();
		else if (GetKey (strLine) == "software_version") m_iSoftwareVersion = GetValue (strLine).toInt();
	}
	if (m_strDBPath.isEmpty() || m_strDBHost.isEmpty() || m_iDBVersion == 0) return false;
	return true;
}
